import io
import os
import unittest
import zipfile
from pathlib import Path
from tarfile import ExFileObject

from drb.exceptions.core import DrbNotImplementationException
from drb.drivers.file import DrbFileFactory
from drb.drivers.tar import DrbTarFactory

SENTINEL_1_ROOT = "sentinel-1"

SENTINEL_1_MANIFEST = "manifest.safe"
SENTINEL_1_SUPPORT = "support"


class TestDrbZip(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    tar_test = current_path / "files" / "test.tar"
    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbTarFactory().create(self.node_file)
        return self.node

    def test_has_impl(self):
        node = self.open_node(str(self.tar_test))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_dir_xml = first_node["xml"]

        node_file_xml_1 = node_dir_xml["a_xml"]

        self.assertTrue(node_file_xml_1.has_impl(ExFileObject))

        self.assertTrue(node_file_xml_1.has_impl(io.BufferedIOBase))

        self.assertTrue(node_file_xml_1.has_impl(io.BufferedReader))

        self.assertFalse(node_dir_xml.has_impl(ExFileObject))

        self.assertFalse(node_dir_xml.has_impl(io.BufferedIOBase))

    def test_get_impl_exception(self):
        node = self.open_node(str(self.tar_test))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_dir_xml = first_node['xml']

        with self.assertRaises(DrbNotImplementationException):
            node_dir_xml.get_impl(io.BufferedIOBase)
        with self.assertRaises(DrbNotImplementationException):
            node_dir_xml.get_impl(ExFileObject)

        node_file_xml_1 = node_dir_xml["a_xml"]
        with self.assertRaises(DrbNotImplementationException):
            node_file_xml_1.get_impl(io.BytesIO)

    def test_get_impl(self):
        node = self.open_node(str(self.tar_test))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_dir_xml = first_node['xml']

        node_file_1 = node_dir_xml["a_xml"]

        self.assertIsNotNone(node_file_1.get_impl(ExFileObject))

        impl = node_file_1.get_impl(ExFileObject)

        self.assertIsInstance(impl, ExFileObject)
        self.assertIsInstance(impl, io.BufferedIOBase)

        impl.close()

    def test_get_impl_read_line(self):
        node = self.open_node(str(self.tar_test))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_dir_xml = first_node["xml"]

        node_file_xml_1 = node_dir_xml["a_xml"]

        self.assertIsNotNone(node_file_xml_1.get_impl(ExFileObject))

        impl = node_file_xml_1.get_impl(io.BufferedIOBase)

        impl.readline()
        impl.readline()
        impl.readline()

        line4 = impl.readline()
        self.assertIn("XML1!!", str(line4))

        impl.close()

    def test_get_impl_read_buffer(self):
        node = self.open_node(str(self.tar_test))

        self.assertIsNotNone(node[0])

        first_node = node[0]

        node_dir_xml = first_node['xml']

        node_file_xml_1 = node_dir_xml["a_xml"]

        self.assertIsNotNone(node_file_xml_1.get_impl(ExFileObject))

        impl = node_file_xml_1.get_impl(ExFileObject)

        impl.seek(20)

        buffer = impl.read(10)
        self.assertIn("XML1!!", str(buffer))

        impl.close()

    def test_file_has_impl(self):
        node = self.open_node(str(self.tar_test))

        self.assertTrue(node.has_impl(io.BufferedIOBase))

        self.assertFalse(node.has_impl(zipfile.ZipExtFile))

    def test_get_file_impl(self):
        node = self.open_node(str(self.tar_test))

        impl = node.get_impl(io.BufferedIOBase)

        self.assertIsNotNone(impl)

        self.assertIsInstance(impl, io.BufferedIOBase)
        impl.close()

    def test_get_file_impl_exception(self):
        node = self.open_node(str(self.tar_test))

        with self.assertRaises(DrbNotImplementationException):
            node.get_impl(zipfile.ZipExtFile)



from deprecated import deprecated
from pathlib import Path
from tarfile import ExFileObject, TarInfo, DIRTYPE
from typing import Any, List
from drb.core.node import DrbNode
from drb.nodes.abstract_node import AbstractNode
from drb.exceptions.core import DrbNotImplementationException
import os
import datetime
import enum
import drb.topics.resolver as resolver


class DrbTarAttributeNames(enum.Enum):
    SIZE = 'size'
    """
    The size of the file in bytes.
    """
    DIRECTORY = 'directory'
    """
    A boolean that tell if the file is a directory.
    """
    MODIFIED = 'modified'
    """
    The last modification date of the file with this format:
        [DAY MONTH NUMB HH:MM:SS YEAR].
    """


class DrbTarNode(AbstractNode):
    """
    This node is used to browse the content of a zip container.

    Parameters:
        parent (DrbNode): The zip container.
        tar_info (ZipInfo): Class with attributes describing
                            each file in the ZIP archive.

    """

    def __init__(self, parent: DrbNode, tar_info: TarInfo = None,
                 name=None, dir=False):
        super().__init__()
        self._tar_info = tar_info
        self.name = name if tar_info is None \
            else self.__init_name(name, tar_info)
        self.parent: DrbNode = parent
        self._children: List[DrbNode] = None
        self._dir = dir if tar_info is None else tar_info.type == DIRTYPE
        self.__init_attributes()
        if not self @ DrbTarAttributeNames.DIRECTORY.value:
            self.add_impl(ExFileObject, _to_stream)

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, item):
        raise NotImplementedError

    def __init_attributes(self):
        name_attribute = DrbTarAttributeNames.DIRECTORY.value
        self @= (name_attribute, None, self._dir)

        if self._tar_info is not None:
            name_attribute = DrbTarAttributeNames.SIZE.value
            self @= (name_attribute, None, self._tar_info.size)

            date_time = datetime.datetime.fromtimestamp(self._tar_info.mtime)
            name_attribute = DrbTarAttributeNames.MODIFIED.value
            self @= (name_attribute, None, date_time.strftime("%c"))

    @staticmethod
    def __init_name(name: str, tar_info: TarInfo) -> str:
        n = name
        if tar_info is not None:
            if tar_info.name.endswith('/'):
                n = tar_info.name[:-1]
            else:
                n = tar_info.name
            if '/' in n:
                return n[n.rindex('/') + 1:]
        return n

    def name_entry(self):
        if self._tar_info is not None:
            return self._tar_info.name
        else:
            name_entry = self.parent.name_entry() + self.name
            if self._dir:
                name_entry += '/'
            return name_entry

    @staticmethod
    def is_a_subdir(entry) -> bool:
        if os.path.basename(entry.name) is not None and len(
                os.path.basename(entry.name)) > 1:
            return False

        paths_array = Path(entry.name).parts

        if len(paths_array) > 1:
            return False
        return True

    def _add_sub_child(self):

        name = self.name_entry()
        for entry in self.get_members():
            filename = entry.name[len(name):]

            # Chek if this entries is a child or a sub
            # child if yes => not a child
            # of the root
            # if os.path.basename(entry.name) is not None and len(
            #         os.path.basename(entry.name)) > 1:
            #         continue

            if filename.startswith('/'):
                filename = filename[1:]

            paths_array = Path(filename).parts

            if len(paths_array) == 1:
                continue

            name_sub_dir = paths_array[0]

            found = False
            for child in self._children:
                if child.name == name_sub_dir:
                    found = True
            if not found:
                self._children.append(DrbTarNode(self, None,
                                                 name=name_sub_dir, dir=True))

    def get_members(self):
        if not self._dir:
            return []

        members = []

        name = self.name_entry()
        if name.startswith('/'):
            name = name[1:]
        for entry in self.parent.get_members():
            entry_name = entry.name
            if entry_name.startswith('/'):
                entry_name = entry_name[1:]
            if not entry_name.startswith(name):
                continue

            filename = entry_name[len(name):]
            if not filename:
                continue

            members.append(entry)
        return members

    def _is_a_child(self, filename):
        name = self.name_entry()
        filename = filename[len(name):]

        if not filename.startswith('/') and \
                not name.endswith('/'):
            return False

        filename = filename[1:]
        if filename.endswith('/'):
            filename = filename[:-1]

        paths_array = Path(filename).parts

        if len(paths_array) > 1:
            return False

        if os.path.basename(filename) is not None \
                and len(os.path.basename(filename)) > 0:
            return True

        # Either the name do not contains sep either only one a last position
        return '/' not in filename

    @property
    @resolver.resolve_children
    @deprecated(version='2.1.0')
    def children(self) -> List[DrbNode]:

        if self._children is None:
            self._children = []

            for entry in self.get_members():
                if self._is_a_child(entry.name):
                    self._children.append(DrbTarNode(self, entry))
                elif self.is_a_subdir(entry):
                    self._children.append(DrbTarNode(self,
                                                     None, name=entry.name))

            self._add_sub_child()

            self._children = sorted(self._children,
                                    key=lambda entry_cmp: entry_cmp.name)

        return self._children

    def open_member(self, tar_info: TarInfo):
        # open a member to retrieve tje implementation
        # back to first parent that is file tar to open it...
        return self.parent.open_member(tar_info)


def _to_stream(node: DrbTarNode, **kwargs):
    return node.parent.open_member(node._tar_info)



import io
import tarfile
from typing import Any, Dict, Tuple, Optional

from drb.core.node import DrbNode
from drb.core.factory import DrbFactory
from drb.core.path import ParsedPath
from drb.exceptions.tar import DrbTarNodeException
from .node import DrbTarNode
from pathlib import Path


class DrbBaseTarNode(DrbTarNode):
    """
    This node is used to open a tar container, and browse his content.

    Parameters:
        base_node (DrbNode): The base node.
    """
    def __init__(self, base_node: DrbNode):
        super().__init__(parent=base_node.parent, tar_info=None)
        self.base_node = base_node
        self.name = self.base_node.name
        self.namespace_uri = self.base_node.namespace_uri
        self.parent = self.base_node.parent
        self.value = self.base_node.value
        self._all_members = None
        self._tar_file = None
        self._tar_file_source = None
        for name, namespace in self.base_node.attribute_names():
            self @= (name, namespace, self.base_node @ (name, namespace))

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError

    @property
    def path(self) -> ParsedPath:
        """
        Returns the path of the base node.

        Returns:
            ParsedPath: the full path of the base node
        """
        return self.base_node.path

    def name_entry(self):
        return ''

    @property
    def tar_file(self) -> tarfile.TarFile:
        if self._tar_file is None:
            try:

                if self.base_node.has_impl(io.BufferedIOBase):
                    self._tar_file_source = self.base_node \
                        .get_impl(io.BufferedIOBase)
                    # If the impl is not seekable
                    # it creates a ByteIO with the entire buff
                    if not self._tar_file_source.seekable():
                        self._tar_file_source = io.BytesIO(
                            self._tar_file_source.read())
                    self._tar_file = tarfile.open(
                        fileobj=self._tar_file_source)
                else:
                    raise DrbTarNodeException(
                        f'Unsupported base_node'
                        f' {type(self.base_node).__name__} '
                        f'for DrbFileTarNode')
            except Exception as e:
                raise DrbTarNodeException(f'Unable to read tar file'
                                          f' {self.name} ') from e

        return self._tar_file

    def has_impl(self, impl: type) -> bool:
        return self.base_node.has_impl(impl)

    def get_impl(self, impl: type, **kwargs) -> Any:
        return self.base_node.get_impl(impl)

    def get_members(self):
        if self._all_members is None:
            self._all_members = self.tar_file.getmembers()
        return self._all_members

    def _is_a_child(self, filename):

        # chek if this entries is a child or a sub child if yes => not a child
        # of the root
        if any(filename.startswith(name_entry) and filename != name_entry
               for name_entry in self.tar_file.getnames()):
            return False

        paths_array = Path(filename).parts

        if len(paths_array) == 1 or (len(paths_array) == 2
                                     and paths_array[0] == '/'):
            return True

        return False

    def _add_sub_child(self):

        for entry in self.get_members():
            filename = entry.name
            # chek if this entries is a child or a sub child
            # if yes => not a child
            # of the root
            if any(filename.startswith(name_entry) and filename != name_entry
                   for name_entry in self.tar_file.getnames()):
                return False

            paths_array = Path(filename).parts

            if len(paths_array) == 1 or (
                    len(paths_array) == 2 and
                    Path(paths_array[0]).as_posix() == '/'):
                continue

            if Path(paths_array[0]).as_posix() == '/':
                name_sub_dir = paths_array[1]
            else:
                name_sub_dir = paths_array[0]

            found = False
            for child in self._children:
                if child.name == name_sub_dir:
                    found = True
            if not found:
                self._children.append(DrbTarNode(self, None,
                                                 name=name_sub_dir,
                                                 dir=True))

    def open_member(self, tar_info: tarfile.TarInfo):
        # open a member of the tar en return an BufferedIOBase impl
        return self._tar_file.extractfile(tar_info)

    def close(self):
        if self._tar_file_source is not None:
            self._tar_file_source.close()
        if self._tar_file is not None:
            self._tar_file.close()
        self.base_node.close()


class DrbTarFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        if isinstance(node, DrbTarNode):
            return node
        return DrbBaseTarNode(base_node=node)



from .base_node import DrbBaseTarNode, DrbTarFactory, DrbTarNode
from . import _version

__version__ = _version.get_versions()['version']
del _version

__all__ = [
    'DrbTarNode',
    'DrbBaseTarNode',
    'DrbTarFactory',
]



import os
import unittest
from pathlib import Path

from drb.exceptions.core import DrbException
from drb.drivers.file import DrbFileFactory
from drb.exceptions.tar import DrbTarNodeException
from drb.drivers.tar import DrbTarFactory
from drb.drivers.tar.node import DrbTarAttributeNames

T__V_T___T__DBL = "S2__OPER_AUX_ECMWFD_PDMC_20190216T120000_" \
                  "V20190217T090000_20190217T210000.DBL"

LONG_DIR = "veryveryveryveryveryveryveryveryveryveryveryveryveryvery" \
           "veryveryveryveryveryveryLongDir"

T____ANNOTATION_DAT = "S2A_OPER_PRD_HKTM___20191121T080247_20191121T080252" \
                      "_0001_annotation.dat"

T____SAFE = "S2A_OPER_PRD_HKTM___20191121T080247_20191121T080252_0001.SAFE"

T__HDR_XML = "S2__OPER_AUX_ECMWFD_PDMC_20190216T120000_V20190217T090000_" \
             "20190217T210000.HDR.xml"


class TestDrbZipFactory(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    tar_test = current_path / "files" / "test.tar"
    tar_gnu = current_path / "files" / "gnu.tar"
    tar_old_gnu = current_path / "files" / "oldgnu.tar"
    tar_posix = current_path / "files" / "posix.tar"
    tar_ustar = current_path / "files" / "ustar.tar"
    tar_v7 = current_path / "files" / "v7.tar"
    tar_file_long_names = current_path / "files" / "test_long_file.tar"
    tar_absolute_path = current_path / "files" / "test_absolute_path.tar"
    tar_relative_path = current_path / "files" / "test_relative_path.tar"
    tar_confuse_name = current_path / "files" / "tar_with_confuse_names.tar"
    tar_fake = current_path / "files" / "fake.tar"
    tar_s2 = current_path / "files" / "S2A_OPER_MSI.tar"
    tar_test_gz = current_path / "files" / "test.tar.gz"

    node = None
    node_file = None

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbTarFactory().create(self.node_file)
        return self.node

    def check_test_tar(self, node):
        self.assertTrue(node.has_child())

        self.assertIsNotNone(node['test'])
        self.assertIsNotNone(node['test']['empty.file'])
        self.assertIsNotNone(node['test']['not-empty.file'])
        self.assertIsNotNone(node['test']["a"])
        self.assertIsNotNone(node['test']["a"]["aaa.txt"])

        with self.assertRaises(KeyError):
            node['test']["aa"]
            node['test']["a"]["aaa.tx"]

        self.assertIsNotNone(node['test']["b"]["bbb.txt"])

        with self.assertRaises(KeyError):
            node['test']["a"]["bbb.tx"]
            node['test']["b"]["aaa.tx"]

    def test_opened_file_node(self):
        node = self.open_node(str(self.tar_test))
        self.assertEqual(node.name, "test.tar")

        self.check_test_tar(node)

    def test_fake(self):
        node = self.open_node(str(self.tar_fake))

        self.assertEqual(node.name, "fake.tar")

        with self.assertRaises(DrbTarNodeException):
            len(node)

    def test_confuse_name(self):
        node = self.open_node(str(self.tar_confuse_name))

        self.check_test_tar(node)

        self.assertEqual(node[0]._tar_info.name, "test")

        self.assertEqual(len(node['test']), 8)

        self.assertIsNotNone(node['test']["xm"])
        self.assertFalse(node['test']["xm"].has_child())
        self.assertTrue(node['test']["xml"].has_child())

        self.assertEqual(len(node['test']["xml"]), 2)

        self.assertIsNotNone(node['test']["not-empty"])
        with self.assertRaises(KeyError):
            node['test']["not-empty", None, 1]
        self.assertFalse(node['test']["not-empty"].has_child())

        self.assertEqual(node[0]._tar_info.name, "test")

    def test_absolute_path(self):
        node = self.open_node(str(self.tar_absolute_path))

        self.assertTrue(node.has_child())

        self.assertEqual(node[0].name, "tmp")
        self.assertTrue(node[0].has_child())
        self.assertEqual(node[0][0].name, "test")
        self.assertTrue(node[0][0].has_child())

        self.assertIsNotNone(node[0][0]['xml']['a.xml'])
        self.assertFalse(node[0][0].has_child('a.xml'))

    def test_relative_path(self):
        node = self.open_node(str(self.tar_relative_path))

        self.assertTrue(node.has_child())

        self.assertEqual(node[0].name, "tmp")
        self.assertEqual(node['tmp'][0].name, "test")
        self.assertEqual(node['tmp'][0][0].name, "a")

        self.check_test_tar(node[0])

    def chack_small_name_tar(self, first_node):
        self.assertTrue(first_node.has_child())
        self.assertIsNotNone(
            first_node["SMALL_NAME"])
        node = first_node["SMALL_NAME"]
        self.assertIsNotNone(node[T__HDR_XML])
        self.assertIsNotNone(node[T__V_T___T__DBL])

        with self.assertRaises(KeyError):
            node[(T__V_T___T__DBL, None, 1)]

        # TODO mix XML and Tar ....
        # node_xml = node.get_named_child(
        # "S2__OPER_AUX_ECMWFD_PDMC_20190216T120000_V20190217T090000"
        # "_20190217T210000.HDR.xml", occurrence=1)
        # self.assertIsNotNone(node_xml.get_named_child(
        # "Earth_Explorer_Header", occurrence=1))

        self.assertEqual(len(node), 2)

        node_xml = node[T__HDR_XML]
        self.assertEqual(node_xml.name,
                         T__HDR_XML)

    def test_gnu_tar(self):
        first_node = self.open_node(str(self.tar_gnu))

        self.chack_small_name_tar(first_node)

    def test_old_gnu_tar(self):
        first_node = self.open_node(str(self.tar_old_gnu))

        self.chack_small_name_tar(first_node)

    def test_posix_tar(self):
        first_node = self.open_node(str(self.tar_posix))

        self.chack_small_name_tar(first_node)

    def test_ustar_tar(self):
        first_node = self.open_node(str(self.tar_ustar))

        self.chack_small_name_tar(first_node)

    def test_v7_tar(self):
        first_node = self.open_node(str(self.tar_v7))

        self.chack_small_name_tar(first_node)

    def test_file_long_names(self):
        first_node = self.open_node(str(self.tar_file_long_names))

        self.assertIsNotNone(first_node[T____SAFE])
        node = first_node[T____SAFE]

        self.assertIsNotNone(node[T____ANNOTATION_DAT])

        self.assertIsNotNone(first_node[T____SAFE])

        node = first_node[T____SAFE]

        for index in range(2):
            self.assertIsNotNone(node[LONG_DIR])
            node = node[LONG_DIR]
            self.assertEqual(node.name, LONG_DIR)

        # self.assertIsNotNone(node.get_named_child("manifest.safe",
        #                                          occurrence=1))

    def test_value(self):
        node = self.open_node(str(self.tar_test))

        self.assertTrue(node[0].has_child())

        self.assertIsNone(node[0].value)

    def test_namespace_uri(self):
        node = self.open_node(str(self.tar_test))

        self.assertTrue(node[0].has_child())

        self.assertIsNone(node[0].namespace_uri)

    def test_namespace_uri_file(self):
        node = self.open_node(str(self.tar_test))

        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)
        self.assertEqual(node.name, self.node_file.name)

    def test_value_file(self):
        node_file = DrbFileFactory().create(str(self.tar_test))
        node = DrbTarFactory().create(node_file)

        self.assertEqual(node.namespace_uri, node_file.namespace_uri)
        self.assertEqual(node.value, node_file.value)

    def test_attributes(self):
        first_node = self.open_node(str(self.tar_test))

        node_dir_xml = first_node['test']['xml']

        list_attributes = node_dir_xml.attributes

        self.assertIn(('directory', None), list_attributes.keys())

        self.assertEqual(True, node_dir_xml @ 'directory')
        self.assertEqual(True, node_dir_xml @ ('directory', None))
        self.assertEqual(True, node_dir_xml.get_attribute('directory'))

        with self.assertRaises(KeyError):
            list_attributes[('directory', 'test')]
        with self.assertRaises(DrbException):
            node_dir_xml @ ('directory', 'test')

    def test_get_attribute(self):
        node = self.open_node(str(self.tar_test))

        node_dir_xml = node['test']['xml']

        self.assertEqual(0, node_dir_xml @ 'size')
        self.assertEqual(0, node_dir_xml @ ('size', None))
        self.assertEqual(0, node_dir_xml.get_attribute('size'))

        self.assertEqual(True, node_dir_xml @ 'directory')
        self.assertEqual(True, node_dir_xml @ ('directory', None))
        self.assertEqual(True, node_dir_xml.get_attribute('directory'))

        actual = node_dir_xml @ 'modified'
        self.assertIn("2017", actual)
        self.assertIn("14", actual)

        node_a_xml = node_dir_xml["a_xml"]
        self.assertEqual(44, node_a_xml.get_attribute('size'))
        self.assertEqual(False, node_a_xml @ 'directory')

    def test_attributes_zip_files(self):
        node = self.open_node(str(self.tar_test))
        self.assertIn(('directory', None), node.attribute_names())
        self.assertIn(('size', None), node.attribute_names())
        self.assertEqual(False, node @ 'directory')
        self.assertEqual(False, node @ ('directory', None))
        self.assertEqual(False, node.get_attribute('directory'))

    def test_parent(self):
        first_node = self.open_node(str(self.tar_test))

        self.assertEqual(first_node.parent, self.node_file.parent)
        self.assertEqual(first_node.value, self.node_file.value)

        first_child = first_node[0]
        self.assertEqual(first_child.parent, first_node)

        first_child.close()

    def test_path(self):
        first_node = self.open_node(str(self.tar_test))

        self.assertEqual(first_node.path.path, self.tar_test.as_posix())

        node_dir_xml = first_node['test']['xml']
        self.assertEqual(node_dir_xml.path.path,
                         self.tar_test.joinpath('test', "xml").as_posix())

    def test_s2(self):

        first_node = self.open_node(str(self.tar_s2))

        self.assertEqual(first_node.path.path, self.tar_s2.as_posix())

        self.assertTrue(first_node.has_child('S2A_OPER_MSI'))
        self.assertEqual(len(first_node), 1)

        node_dir_qi = first_node['S2A_OPER_MSI']['QI_DATA']

        self.assertIn(('directory', None), node_dir_qi.attribute_names())
        self.assertEqual(True, node_dir_qi @ 'directory')

        self.assertTrue('S2A_OPER_MSI_report.xml' in node_dir_qi)
        self.assertFalse(
            'S2A_OPER_MSI_report.xml' in first_node['S2A_OPER_MSI'])

        self.assertEqual(
            False, node_dir_qi['S2A_OPER_MSI_report.xml'] @ 'directory')

    def test_tar_gz(self):
        node = self.open_node(str(self.tar_test_gz))

        self.assertTrue(node[0].has_child())

        self.assertIsNone(node[0].value)



import os
import unittest
import uuid
from pathlib import Path

from drb.core.factory import FactoryLoader
from drb.topics.topic import TopicCategory
from drb.topics.dao import ManagerDao
from drb.nodes.logical_node import DrbLogicalNode

from drb.drivers.tar import DrbTarFactory


class TestDrbTarSignature(unittest.TestCase):
    mock_pkg = None
    fc_loader = None
    ic_loader = None

    tar_id = uuid.UUID('60e58ee2-2b5c-11ec-8d3d-0242ac130003')
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    tar_test = current_path / "files" / "test.tar"
    tar_gz_test = current_path / "files" / "test.tar.gz"
    empty_file = current_path / "files" / "empty.file"

    @classmethod
    def setUpClass(cls) -> None:
        cls.fc_loader = FactoryLoader()
        cls.ic_loader = ManagerDao()

    def test_impl_loading(self):
        factory_name = 'tar'

        factory = self.fc_loader.get_factory(factory_name)
        self.assertIsNotNone(factory)
        self.assertIsInstance(factory, DrbTarFactory)

        item_class = self.ic_loader.get_drb_topic(self.tar_id)
        self.assertIsNotNone(factory)
        self.assertEqual(self.tar_id, item_class.id)
        self.assertEqual('tar', item_class.label)
        self.assertIsNone(item_class.description)
        self.assertEqual(TopicCategory.CONTAINER, item_class.category)
        self.assertEqual('tar', item_class.factory)

    def test_impl_signatures(self):
        item_class = self.ic_loader.get_drb_topic(self.tar_id)

        node = DrbLogicalNode(self.tar_test)
        self.assertTrue(item_class.matches(node))

        node = DrbLogicalNode(self.tar_gz_test)
        self.assertTrue(item_class.matches(node))

        node = DrbLogicalNode(self.empty_file)
        self.assertFalse(item_class.matches(node))



import versioneer
from setuptools import setup

setup(
    test_suite='tests',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    project_urls={
        'Documentation': 'https://drb-python.gitlab.io/impl/tar',
        'Source': 'https://gitlab.com/drb-python/impl/tar',
    }
)
